<?php
/**
* Plugin Name: EU VAT Reports for Woocommerce
* Description: Easily generate VAT reports required by the European countries on the basis of country filter.  You can download the report in CSV format too.
* Version: 1.1.2
* Author: Simple Intelligent Systems
* Author URI: https://simpleintelligentsystems.com/
* License: GPLv2 or later
*/

/**
 * Enqueue style and scripts to plugin admin section
 */
function simpleIntelligentsystems_eu_vat_style() {
	//CSS
    wp_enqueue_style( 'eu-vat-reports-for-woocommerce-style',  plugin_dir_url( __FILE__ ) . 'css/style.css' );
    
	//Javascript
	wp_enqueue_script( 'eu-vat-reports-for-woocommerce-script',  plugin_dir_url( __FILE__ ) . 'js/script.js', array(), '', true );
	
	// Pass ajax_url to script.js
	wp_localize_script( 'eu-vat-reports-for-woocommerce-script', 'euvat_ajax_data', array( 'ajax_url' => admin_url( 'admin-ajax.php' ) ) );
}
add_action( 'admin_enqueue_scripts', 'simpleIntelligentsystems_eu_vat_style' );


/**
 * Create seperate menu for EU VAT Report
 */
function simpleIntelligentsystems_eu_vat_report_menu() {
	if ( class_exists( 'WooCommerce' ) ) {  
    	add_menu_page(
        	__( 'European Countries Orders', 'sis-export-orders' ),
        	__( 'EU Country Orders', 'sis-export-orders' ),
        	'manage_options',
        	'woo-export-orders',
        	'simpleIntelligentsystems_eu_vat_report_contents',
        	'dashicons-schedule',
        	55
    	);
    }
    else {
    	add_action( 'admin_notices', 'simpleIntelligentsystems_eu_vat_report_notice' );
    	return false; 
    }
}
add_action( 'init', 'simpleIntelligentsystems_eu_vat_report_menu' );

/**
 * Display notice
 */
function simpleIntelligentsystems_eu_vat_report_notice() {
  echo "<div class='error'><p>EU VAT Reports for Woocommerce requires WooCommerce, Please activate WooCommerce!</p></div>";
}

/**
 * Create function to display form with different selection fields and list orders based on selection items
 */
function simpleIntelligentsystems_eu_vat_report_contents() { ?>
    <h1> <?php esc_html_e( 'List of European Countries Orders', 'woo-export-orders' ); ?> </h1>
    <?php 
	session_start();
    $shipping_country_list = array( 
                                    'AT' => 'Austria',
                                	'BE' => 'Belgium', 
                                	'BG' => 'Bulgaria', 
                                	'HR' => 'Croatia', 
                                	'CY' => 'Cyprus', 
                                	'DK' => 'Denmark', 
                                	'EE' => 'Estonia', 
                                	'FI' => 'Finland', 
                                	'FR' => 'France', 
                                	'DE' => 'Germany', 
                                	'GR' => 'Greece', 
                                	'HU' => 'Hungary', 
                                	'HR' => 'Ireland', 
                                	'IT' => 'Italy', 
                                	'LV' => 'Latvia', 
                                	'LT' => 'Lithuania', 
                                	'LU' => 'Luxembourg', 
                                	'MT' => 'Malta', 
                                	'NL' => 'Netherlands', 
                                	'PL' => 'Poland', 
                                	'PT' => 'Portugal', 
                                	'RO' => 'Romania', 
                                	'SK' => 'Slovakia', 
                                	'ES' => 'Spain', 
                                	'SE' => 'Sweden'                                
                                );
	
	if(isset($_POST['shipping_country'])){
    	$_SESSION['shipping_country'] = sanitize_text_field($_POST['shipping_country']);
    	$selected_country_via_post = sanitize_text_field($_SESSION['shipping_country']);
	}
	if(isset($_POST['from_date'])){
		$_SESSION['from_date'] = sanitize_text_field($_POST['from_date']);
    	$from_date = sanitize_text_field($_SESSION['from_date']);
	}
	if(isset($_POST['to_date'])){
		$_SESSION['to_date'] = sanitize_text_field($_POST['to_date']);
    	$to_date = sanitize_text_field($_SESSION['to_date']);
	}
	if(isset($_SESSION['shipping_country'])){
    	$selected_country_via_post = sanitize_text_field($_SESSION['shipping_country']);
	}
	if(isset($_SESSION['from_date'])){
    	$from_date = sanitize_text_field($_SESSION['from_date']);
	}
	if(isset($_SESSION['to_date'])){
    	$to_date = sanitize_text_field($_SESSION['to_date']);
	}
	$url = esc_url_raw(strtok($_SERVER["REQUEST_URI"], '?'));
    ?>
    <form method="POST" action="<?php echo esc_url($url); ?>?page=woo-export-orders" id="export_form">
    	
        <label for="shipping_country">Select country</label>
        <select name="shipping_country" id="shipping_country">
        <?php 
		foreach( $shipping_country_list as $list_key => $list_value ){
        ?>
			<option value="<?php echo esc_html($list_key); ?>" <?php echo esc_html(($selected_country_via_post=="$list_key") || (( $selected_country_via_post=="") && ( $list_key=="NL")) ?"selected":""); ?>><?php echo esc_html($list_value); ?></option>
		<?php
        } 
		?>
        </select>
        
        <span class="input-span">From Date:</span> <input type="date" name="from_date" value="<?php if( isset($from_date) ){ echo esc_attr($from_date); } ?>"/>
        <span class="input-span">To Date:</span> <input type="date" name="to_date" value="<?php if( isset($to_date) ){ echo esc_attr($to_date); } ?>"/>
        
        <input type="submit" value="Submit" name="submit">
    </form>  
    <a href="javascript:void(0)" id="backend_downloadcsv">Download CSV</a>
    <?php
    
    $selected_country = 'NL'; 
	if( isset($_POST['submit']) || $_SESSION['shipping_country'] ){ 
	    $selected_country = $selected_country_via_post;
	}

	global $wpdb;
    $tablename = $wpdb->prefix.'posts';
    $post_status_array= "( 'wc-processing', 'wc-completed') ";

    $customPagHTML  = "";
    $query          = "SELECT po.id, po.post_status,pm.meta_key,pm.meta_value FROM $tablename po INNER JOIN wp_postmeta pm on po.id = pm.post_id where po.post_type='shop_order' and po.post_status in ".$post_status_array." and pm.meta_key='_shipping_country' and pm.meta_value='".$selected_country."'";
	if( !empty($from_date) && !empty($to_date) ){
    	$query		.= " and post_date between '".$from_date."' and '".$to_date."'";	
    }else if( !empty($from_date) && empty($to_date) ){
    	$query		.= " and post_date > '".$from_date."'";	
    }else if( empty($from_date) && !empty($to_date) ){
    	$query		.= " and post_date < '".$to_date."'";	
    }
	
    $total_query    = "SELECT COUNT(1) FROM (${query}) AS combined_table";
    $total          = $wpdb->get_var( $total_query );
    $items_per_page = 20;
    $page           = isset( $_GET['cpage'] ) ? abs( (int) $_GET['cpage'] ) : 1;
    $offset         = ( $page * $items_per_page ) - $items_per_page;
    $totalPage      = ceil($total / $items_per_page);
    if( $totalPage > 1 )
    {
        $customPagHTML     =  '<div class="pagination"><span>Page '.$page.' of '.$totalPage.'</span>&nbsp;&nbsp;'.paginate_links( array(
        'base' => add_query_arg( 'cpage', '%#%' ),
        'format' => '',
        'prev_text' => __('&laquo;'),
        'next_text' => __('&raquo;'),
        'total' => $totalPage,
        'current' => $page
        )).'</div>';
    }

    $q = $query . " ORDER BY ID DESC LIMIT ${offset}, ${items_per_page}";
	
    $result         = $wpdb->get_results( $q, ARRAY_A );

	$taxRateQuery   = "SELECT tax_rate FROM wp_woocommerce_tax_rates where tax_rate_country='".$selected_country."'";
	$taxRateResult  = $wpdb->get_row( $taxRateQuery );
	$taxRate = ceil( $taxRateResult->tax_rate );
	 
    $tableStructure =   "<table width=\"100%\" border=\"1\" cellspacing=\"0\" cellpadding=\"5\" ><thead>
                         <tr>
                            <th>Order Date</th>
                            <th>Order ID/Invoice #</th>
                            <th>Order Status</th>
                            <th>Customer Name</th>
                            <th>Order Amount</th>
                            <th>Payment Mode</th>
                            <th>VAT Amount</th>
                         </tr>
                        </thead>";

    if( $result )
    {
        $tableStructure .= "<tbody>";                   
        foreach( $result as $row )
        {
        	$order = new WC_Order($row['id']);
        	
        	$order_total = $order->get_total();
			$order_date = $order->order_date;
        	$user_id = $order->get_user_id( );
        	$customer = new WC_Customer( $user_id );
        	$username = $user_id>0?$first_name   = $customer->get_first_name().' '.$last_name    = $customer->get_last_name() : 'Guest User';
      		
        	// Get gift card amount if used in an order
			$gift_card_order_query   = "SELECT order_item_id FROM ".$wpdb->prefix."woocommerce_order_items where order_item_type='pw_gift_card' and order_id=".$row['id'];
        	$gift_card_order_data  = $wpdb->get_row( $gift_card_order_query );
        	$amount = 0.0;
        	if($gift_card_order_data){
        		$gift_card_discount_query = "SELECT * FROM ".$wpdb->prefix."woocommerce_order_itemmeta where meta_key='amount' and order_item_id=".$gift_card_order_data->order_item_id;
            	$gift_card_discount_result	= $wpdb->get_results( $gift_card_discount_query, ARRAY_A );	
            	foreach( $gift_card_discount_result as $gift_card_row ){
                	$amount += $gift_card_row['meta_value'];
                }
            	$order_total += $amount;
            }
        	
            $poststatus = $row['post_status']== 'wc-processing' ? 'Processing' : 'Completed';
            $tableStructure .= "<tr>";
            $tableStructure .= "<td>".$order_date."</td>";
        	$tableStructure .= "<td>".$row['id']."</td>";
            $tableStructure .= "<td>".$poststatus."</td>";
            $tableStructure .= "<td>".$username."</td>";
            $tableStructure .= "<td>".$order->get_currency().' '.number_format( (float) $order_total, 2, '.', '')."</td>";
            $tableStructure .= "<td>".$order->get_payment_method_title()."</td>";
        	$tableStructure .= "<td>".$order->get_currency().' '.$order->get_total_tax()."</td>";
            $tableStructure .= "</tr>";
        }
        $tableStructure .= "</tbody>";
    	
    	$tableStructure .= "</table>";

		//Modifying the parent query to remove limit and find the total of total result count
		$query_without_limit = strstr($q, 'LIMIT', true);
		$order_total_calculate_nolimit = 0;
        $order_tax_calculate_nolimit = 0;
		$withoutLimitResult = $wpdb->get_results( $query_without_limit, ARRAY_A );
		if( $withoutLimitResult ){
			foreach( $withoutLimitResult as $withoutLimitRow ){
				$noLimitOrder = new WC_Order($withoutLimitRow['id']);
				$order_total_calculate_nolimit += $noLimitOrder->get_total();
        		$order_tax_calculate_nolimit += $noLimitOrder->get_total_tax();
			}
			$netAmount = $order_total_calculate_nolimit - $order_tax_calculate_nolimit;
		}

    	echo '<div class="wrap eu-vat-report-sis">';
    	echo '<h4>VAT % for the selected country: '.esc_attr($taxRate).'%</h4>';
    	echo '<h3>Orders List</h3>';
    	echo wp_kses_post($tableStructure);
    	echo wp_kses_post($customPagHTML);
    	echo '</div>'; 
		echo "<div class='eu-vat-report-final-result'><table>";
		echo "<tr><th>Gross Amount</th> <td>EUR ".esc_attr(number_format($order_total_calculate_nolimit,2))."</td></tr>";
		echo "<tr><th>Net Amount</th> <td>EUR ".esc_attr(number_format($netAmount,2))."</td></tr>";
		echo "<tr><th>VAT Rate</th><td> ".esc_attr($taxRate)."%</td></tr>";
		echo "<tr><th>Total VAT Amount</th> <td>EUR ".esc_attr(number_format($order_tax_calculate_nolimit,2))."</td></tr></table></div>";
    }else{
    	echo "<br><br><h3>No Orders Found.<h3>";
    }
}

/**
 * Ajax request
 */
add_action( 'wp_ajax_euOrderPrintCSV', 'simpleIntelligentsystems_get_data' );

function simpleIntelligentsystems_get_data(){
	global $wpdb;
	$form_params = array();
	parse_str($_POST['values'], $form_params);
	$selected_country = $form_params['shipping_country'];
	$tablename = $wpdb->prefix.'posts';
	$post_status_array= "('wc-processing', 'wc-completed')";
	$query          = "SELECT po.id, po.post_status,pm.meta_key,pm.meta_value FROM $tablename po INNER JOIN wp_postmeta pm on po.id = pm.post_id where po.post_type='shop_order' and po.post_status in ".$post_status_array." and pm.meta_key='_shipping_country' and pm.meta_value='".$selected_country."'";
	if(!empty($form_params['from_date']) && !empty($form_params['to_date'])){
    	$query		.= " and post_date between '".$form_params['from_date']."' and '".$form_params['to_date']."'";	
	}else if(!empty($form_params['from_date']) && empty($form_params['to_date'])){
    	$query		.= " and post_date > '".$form_params['from_date']."'";	
	}else if(empty($form_params['from_date']) && !empty($form_params['to_date'])){
    	$query		.= " and post_date < '".$form_params['to_date']."'";	
	}

	$total_query    = "SELECT COUNT(1) FROM (${query}) AS combined_table";
	$total          = $wpdb->get_var( $total_query );
	$q = $query . " ORDER BY ID DESC";
	$result         = $wpdb->get_results( $q, ARRAY_A );
	$data_array = array();
	foreach($result as $row)
	{
    	$order = new WC_Order($row['id']);
    	$order_date = date("Y-m-d", strtotime($order->order_date));
    	$user_id = $order->get_user_id( );
    	$customer = new WC_Customer( $user_id );
    	$username = $user_id>0?$first_name   = $customer->get_first_name().' '.$last_name    = $customer->get_last_name() : 'Guest User';
    	$poststatus = $row['post_status']== 'wc-processing' ? 'Processing' : 'Completed';
    	$order_total_calculate += $order->get_total();
    	$order_tax_calculate += $order->get_total_tax();
    	$data_array[] = array('Order ID'=>$row['id'],'Order Date'=>$order_date,'Order Status'=>$poststatus,'Customer Name'=>$username,'Order Amount'=>$order->get_currency().' '.$order->get_total(),'Payment Method'=>$order->get_payment_method_title(),'VAT Amount'=>$order->get_currency().' '.$order->get_total_tax());
    }
	return simpleIntelligentsystems_download_csv( $data_array );
	wp_die(); 
}

function simpleIntelligentsystems_download_csv( $assocDataArray ) {
	if ( !empty( $assocDataArray ) ):
	$fp = fopen( 'php://output', 'w' );
	fputcsv( $fp, array_keys( reset($assocDataArray) ) );
	foreach ( $assocDataArray AS $values ):
		fputcsv( $fp, $values );
	endforeach;
	fclose( $fp );
	endif;
	exit();
}
